@@ -80,7 +80,7 @@ main(int argc, char *argv[])
 	struct cpio _cpio; /* Allocated on stack. */
 	struct cpio *cpio;
 	int uid, gid;
-	char opt;
+	int opt;
 
 	cpio = &_cpio;
 	memset(cpio, 0, sizeof(*cpio));
@@ -177,7 +177,7 @@ main(int argc, char *argv[])
 			cpio->verbose++;
 			break;
 		case OPTION_VERSION: /* GNU convention */
-			version(stderr);
+			version(stdout);
 			break;
 #if 0
 	        /*
