@@ -25,6 +25,9 @@
  * $FreeBSD$
  */
 
+#ifndef CPIO_H_INCLUDED
+#define CPIO_H_INCLUDED
+
 #include "cpio_platform.h"
 #include <stdio.h>
 
@@ -46,28 +49,50 @@ struct cpio {
 	const char	 *format; /* -H format */
 	int		  bytes_per_block; /* -b block_size */
 	int		  verbose;   /* -v */
+	int		  quiet;   /* --quiet */
 	int		  extract_flags; /* Flags for extract operation */
 	char		  symlink_mode; /* H or L, per BSD conventions */
 	const char	 *compress_program;
-	int		  option_null; /* -0 --null */
-	int		  option_list; /* -t */
+	int		  option_atime_restore; /* -a */
+	int		  option_follow_links; /* -L */
 	int		  option_link; /* -l */
+	int		  option_list; /* -t */
+	int		  option_null; /* -0 --null */
+	int		  option_rename; /* -r */
+	char		 *pass_destdir;
+	size_t		  pass_destpath_alloc;
+	char		 *pass_destpath;
+	int		  uid_override;
+	int		  gid_override;
 
 	/* If >= 0, then close this when done. */
 	int		  fd;
 
 	/* Miscellaneous state information */
 	struct archive	 *archive;
-	const char	 *progname;
 	int		  argc;
 	char		**argv;
 	int		  return_value; /* Value returned by main() */
+	struct archive_entry_linkresolver *linkresolver;
+
+	struct matching  *matching;
 };
 
-void	cpio_errc(struct cpio *, int _eval, int _code,
-	    const char *fmt, ...);
-void	cpio_warnc(struct cpio *, int _code, const char *fmt, ...);
-void	cpio_strmode(struct archive_entry *, char *);
-int	process_lines(struct cpio *cpio, const char *pathname,
-	    int (*process)(struct cpio *, const char *));
+/* Name of this program; used in error reporting, initialized in main(). */
+const char *cpio_progname;
 
+void	cpio_errc(int _eval, int _code, const char *fmt, ...);
+void	cpio_warnc(int _code, const char *fmt, ...);
+
+int	owner_parse(const char *, int *, int *);
+
+
+/* Fake short equivalents for long options that otherwise lack them. */
+enum {
+	OPTION_QUIET = 1,
+	OPTION_VERSION
+};
+
+int	cpio_getopt(struct cpio *cpio);
+
+#endif
