@@ -79,19 +79,19 @@ DEFINE_TEST(test_format_newc)
 	 */
 	list = open("list", O_CREAT | O_WRONLY, 0644);
 
-	/* "file" */
-	fd = open("file", O_CREAT | O_WRONLY, 0644);
+	/* "file1" */
+	fd = open("file1", O_CREAT | O_WRONLY, 0644);
 	assert(fd >= 0);
 	assertEqualInt(10, write(fd, "123456789", 10));
 	close(fd);
-	assertEqualInt(5, write(list, "file\n", 5));
+	assertEqualInt(6, write(list, "file1\n", 6));
 
 	/* "hardlink" */
-	assertEqualInt(0, link("file", "hardlink"));
+	assertEqualInt(0, link("file1", "hardlink"));
 	assertEqualInt(9, write(list, "hardlink\n", 9));
 
 	/* "symlink" */
-	assertEqualInt(0, symlink("file", "symlink"));
+	assertEqualInt(0, symlink("file1", "symlink"));
 	assertEqualInt(8, write(list, "symlink\n", 8));
 
 	/* "dir" */
@@ -120,7 +120,7 @@ DEFINE_TEST(test_format_newc)
 	 * a little tricky because they depend on the local environment.
 	 */
 
-	/* First entry is "file" */
+	/* First entry is "file1" */
 	assert(is_hex(e, 110)); /* Entire header is octal digits. */
 	assertEqualMem(e + 0, "070701", 6); /* Magic */
 	ino = from_hex(e + 6, 8); /* ino */
@@ -146,14 +146,14 @@ DEFINE_TEST(test_format_newc)
 	devminor = from_hex(e + 70, 8); /* devminor */
 	assert(is_hex(e + 78, 8)); /* rdevmajor */
 	assert(is_hex(e + 86, 8)); /* rdevminor */
-	assertEqualMem(e + 94, "00000005", 8); /* Name size */
+	assertEqualMem(e + 94, "00000006", 8); /* Name size */
 	assertEqualInt(0, from_hex(e + 102, 8)); /* check field */
-	assertEqualMem(e + 110, "file\0\0", 6); /* Name contents */
+	assertEqualMem(e + 110, "file1\0", 6); /* Name contents */
 	/* Since there's another link, no file contents here. */
 	/* But add in file size so that an error here doesn't cascade. */
-	e += 116 + from_hex(e + 54, 8) + (3 & (4 - from_hex(e + 54, 8)));
+	e += 116 + from_hex(e + 54, 8) + (3 & -from_hex(e + 54, 8));
 
-	/* Hardlink identical to "file" */
+	/* Hardlink identical to "file1" */
 	assert(is_hex(e, 110));
 	assertEqualMem(e + 0, "070701", 6); /* Magic */
 	assertEqualInt(ino, from_hex(e + 6, 8)); /* ino */
@@ -171,9 +171,9 @@ DEFINE_TEST(test_format_newc)
 	assertEqualInt(0, from_hex(e + 102, 8)); /* check field */
 	assertEqualMem(e + 110, "hardlink\0\0", 10); /* Name contents */
 	assertEqualMem(e + 120, "123456789\0\0\0", 12); /* File contents */
-	e += 120 + from_hex(e + 54, 8) + (3 & (4 - from_hex(e + 54, 8)));
+	e += 120 + from_hex(e + 54, 8) + (3 & -from_hex(e + 54, 8));
 
-	/* "symlink" pointing to "file" */
+	/* "symlink" pointing to "file1" */
 	assert(is_hex(e, 110));
 	assertEqualMem(e + 0, "070701", 6); /* Magic */
 	assert(is_hex(e + 6, 8)); /* ino */
@@ -182,7 +182,7 @@ DEFINE_TEST(test_format_newc)
 	assertEqualInt(gid, from_hex(e + 30, 8)); /* gid */
 	assertEqualMem(e + 38, "00000001", 8); /* nlink */
 	assertEqualInt(t, from_hex(e + 46, 8)); /* mtime */
-	assertEqualMem(e + 54, "00000004", 8); /* File size */
+	assertEqualMem(e + 54, "00000005", 8); /* File size */
 	assertEqualInt(devmajor, from_hex(e + 62, 8)); /* devmajor */
 	assertEqualInt(devminor, from_hex(e + 70, 8)); /* devminor */
 	assert(is_hex(e + 78, 8)); /* rdevmajor */
@@ -190,8 +190,8 @@ DEFINE_TEST(test_format_newc)
 	assertEqualMem(e + 94, "00000008", 8); /* Name size */
 	assertEqualInt(0, from_hex(e + 102, 8)); /* check field */
 	assertEqualMem(e + 110, "symlink\0\0\0", 10); /* Name contents */
-	assertEqualMem(e + 120, "file", 4); /* symlink target */
-	e += 120 + from_hex(e + 54, 8) + (3 & (4 - from_hex(e + 54, 8)));
+	assertEqualMem(e + 120, "file1\0\0\0", 8); /* symlink target */
+	e += 120 + from_hex(e + 54, 8) + (3 & -from_hex(e + 54, 8));
 
 	/* "dir" */
 	assert(is_hex(e, 110));
