@@ -32,7 +32,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read.c,v 1.35 2007/05/29 01:00:18 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read.c,v 1.37 2008/01/03 17:54:26 des Exp $");
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -306,6 +306,18 @@ archive_read_next_header(struct archive *_a, struct archive_entry **entryp)
 	archive_entry_clear(entry);
 	archive_clear_error(&a->archive);
 
+	/*
+	 * If no format has yet been chosen, choose one.
+	 */
+	if (a->format == NULL) {
+		slot = choose_format(a);
+		if (slot < 0) {
+			a->archive.state = ARCHIVE_STATE_FATAL;
+			return (ARCHIVE_FATAL);
+		}
+		a->format = &(a->formats[slot]);
+	}
+
 	/*
 	 * If client didn't consume entire data, skip any remainder
 	 * (This is especially important for GNU incremental directories.)
@@ -324,12 +336,6 @@ archive_read_next_header(struct archive *_a, struct archive_entry **entryp)
 	/* Record start-of-header. */
 	a->header_position = a->archive.file_position;
 
-	slot = choose_format(a);
-	if (slot < 0) {
-		a->archive.state = ARCHIVE_STATE_FATAL;
-		return (ARCHIVE_FATAL);
-	}
-	a->format = &(a->formats[slot]);
 	ret = (a->format->read_header)(a, entry);
 
 	/*
@@ -735,3 +741,14 @@ __archive_read_register_compression(struct archive_read *a,
 	__archive_errx(1, "Not enough slots for compression registration");
 	return (NULL); /* Never actually executed. */
 }
+
+/* used internally to simplify read-ahead */
+const void *
+__archive_read_ahead(struct archive_read *a, size_t len)
+{
+	const void *h;
+
+	if ((a->decompressor->read_ahead)(a, &h, len) < (ssize_t)len)
+		return (NULL);
+	return (h);
+}
