@@ -1,13 +1,12 @@
 /*-
- * Copyright (c) 2003-2004 Tim Kientzle
+ * Copyright (c) 2003-2007 Tim Kientzle
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer
- *    in this position and unchanged.
+ *    notice, this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
@@ -65,15 +64,24 @@
 
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_compression_compress.c,v 1.5 2006/07/30 00:29:00 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_compression_compress.c,v 1.10 2007/05/29 01:00:19 kientzle Exp $");
 
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
 #include "archive.h"
 #include "archive_private.h"
+#include "archive_read_private.h"
 
 /*
  * Because LZW decompression is pretty simple, I've just implemented
@@ -126,17 +134,20 @@ struct private_data {
 };
 
 static int	bid(const void *, size_t);
-static int	finish(struct archive *);
-static int	init(struct archive *, const void *, size_t);
-static ssize_t	read_ahead(struct archive *, const void **, size_t);
-static ssize_t	read_consume(struct archive *, size_t);
-static int	getbits(struct archive *, struct private_data *, int n);
-static int	next_code(struct archive *a, struct private_data *state);
+static int	finish(struct archive_read *);
+static int	init(struct archive_read *, const void *, size_t);
+static ssize_t	read_ahead(struct archive_read *, const void **, size_t);
+static ssize_t	read_consume(struct archive_read *, size_t);
+static int	getbits(struct archive_read *, struct private_data *, int n);
+static int	next_code(struct archive_read *a, struct private_data *state);
 
 int
-archive_read_support_compression_compress(struct archive *a)
+archive_read_support_compression_compress(struct archive *_a)
 {
-	return (__archive_read_register_compression(a, bid, init));
+	struct archive_read *a = (struct archive_read *)_a;
+	if (__archive_read_register_compression(a, bid, init) != NULL)
+		return (ARCHIVE_OK);
+	return (ARCHIVE_FATAL);
 }
 
 /*
@@ -155,7 +166,7 @@ bid(const void *buff, size_t len)
 	if (len < 1)
 		return (0);
 
-	buffer = buff;
+	buffer = (const unsigned char *)buff;
 	bits_checked = 0;
 	if (buffer[0] != 037)	/* Verify first ID byte. */
 		return (0);
@@ -180,42 +191,42 @@ bid(const void *buff, size_t len)
  * Setup the callbacks.
  */
 static int
-init(struct archive *a, const void *buff, size_t n)
+init(struct archive_read *a, const void *buff, size_t n)
 {
 	struct private_data *state;
 	int code;
 
-	a->compression_code = ARCHIVE_COMPRESSION_COMPRESS;
-	a->compression_name = "compress (.Z)";
+	a->archive.compression_code = ARCHIVE_COMPRESSION_COMPRESS;
+	a->archive.compression_name = "compress (.Z)";
 
-	a->compression_read_ahead = read_ahead;
-	a->compression_read_consume = read_consume;
-	a->compression_skip = NULL; /* not supported */
-	a->compression_finish = finish;
+	a->decompressor->read_ahead = read_ahead;
+	a->decompressor->consume = read_consume;
+	a->decompressor->skip = NULL; /* not supported */
+	a->decompressor->finish = finish;
 
-	state = malloc(sizeof(*state));
+	state = (struct private_data *)malloc(sizeof(*state));
 	if (state == NULL) {
-		archive_set_error(a, ENOMEM,
+		archive_set_error(&a->archive, ENOMEM,
 		    "Can't allocate data for %s decompression",
-		    a->compression_name);
+		    a->archive.compression_name);
 		return (ARCHIVE_FATAL);
 	}
 	memset(state, 0, sizeof(*state));
-	a->compression_data = state;
+	a->decompressor->data = state;
 
 	state->uncompressed_buffer_size = 64 * 1024;
 	state->uncompressed_buffer = malloc(state->uncompressed_buffer_size);
 
 	if (state->uncompressed_buffer == NULL) {
-		archive_set_error(a, ENOMEM,
+		archive_set_error(&a->archive, ENOMEM,
 		    "Can't allocate %s decompression buffers",
-		    a->compression_name);
+		    a->archive.compression_name);
 		goto fatal;
 	}
 
-	state->next_in = buff;
+	state->next_in = (const unsigned char *)buff;
 	state->avail_in = n;
-	state->read_next = state->next_out = state->uncompressed_buffer;
+	state->read_next = state->next_out = (unsigned char *)state->uncompressed_buffer;
 	state->avail_out = state->uncompressed_buffer_size;
 
 	code = getbits(a, state, 8);
@@ -228,7 +239,7 @@ init(struct archive *a, const void *buff, size_t n)
 		 * blocks and gzip and compress are both enabled.
 		 * You can't distinguish gzip and compress only from
 		 * the first byte. */
-		archive_set_error(a, ARCHIVE_ERRNO_FILE_FORMAT,
+		archive_set_error(&a->archive, ARCHIVE_ERRNO_FILE_FORMAT,
 		    "Compress signature did not match.");
 		goto fatal;
 	}
@@ -263,15 +274,15 @@ fatal:
  * as necessary.
  */
 static ssize_t
-read_ahead(struct archive *a, const void **p, size_t min)
+read_ahead(struct archive_read *a, const void **p, size_t min)
 {
 	struct private_data *state;
-	int read_avail, was_avail, ret;
+	size_t read_avail;
+	int ret;
 
-	state = a->compression_data;
-	was_avail = -1;
+	state = (struct private_data *)a->decompressor->data;
 	if (!a->client_reader) {
-		archive_set_error(a, ARCHIVE_ERRNO_PROGRAMMER,
+		archive_set_error(&a->archive, ARCHIVE_ERRNO_PROGRAMMER,
 		    "No read callback is registered?  "
 		    "This is probably an internal programming error.");
 		return (ARCHIVE_FATAL);
@@ -279,22 +290,22 @@ read_ahead(struct archive *a, const void **p, size_t min)
 
 	read_avail = state->next_out - state->read_next;
 
-	if (read_avail < (int)min  &&  state->end_of_stream) {
+	if (read_avail < min  &&  state->end_of_stream) {
 		if (state->end_of_stream == ARCHIVE_EOF)
 			return (0);
 		else
 			return (-1);
 	}
 
-	if (read_avail < (int)min) {
+	if (read_avail < min) {
 		memmove(state->uncompressed_buffer, state->read_next,
 		    read_avail);
-		state->read_next = state->uncompressed_buffer;
+		state->read_next = (unsigned char *)state->uncompressed_buffer;
 		state->next_out = state->read_next + read_avail;
 		state->avail_out
 		    = state->uncompressed_buffer_size - read_avail;
 
-		while (read_avail < (int)state->uncompressed_buffer_size
+		while (read_avail < state->uncompressed_buffer_size
 			&& !state->end_of_stream) {
 			if (state->stackp > state->stack) {
 				*state->next_out++ = *--state->stackp;
@@ -318,12 +329,12 @@ read_ahead(struct archive *a, const void **p, size_t min)
  * Mark a previously-returned block of data as read.
  */
 static ssize_t
-read_consume(struct archive *a, size_t n)
+read_consume(struct archive_read *a, size_t n)
 {
 	struct private_data *state;
 
-	state = a->compression_data;
-	a->file_position += n;
+	state = (struct private_data *)a->decompressor->data;
+	a->archive.file_position += n;
 	state->read_next += n;
 	if (state->read_next > state->next_out)
 		__archive_errx(1, "Request to consume too many "
@@ -335,12 +346,12 @@ read_consume(struct archive *a, size_t n)
  * Clean up the decompressor.
  */
 static int
-finish(struct archive *a)
+finish(struct archive_read *a)
 {
 	struct private_data *state;
 	int ret = ARCHIVE_OK;
 
-	state = a->compression_data;
+	state = (struct private_data *)a->decompressor->data;
 
 	if (state != NULL) {
 		if (state->uncompressed_buffer != NULL)
@@ -348,10 +359,7 @@ finish(struct archive *a)
 		free(state);
 	}
 
-	a->compression_data = NULL;
-	if (a->client_closer != NULL)
-		ret = (a->client_closer)(a, a->client_data);
-
+	a->decompressor->data = NULL;
 	return (ret);
 }
 
@@ -361,7 +369,7 @@ finish(struct archive *a)
  * format error, ARCHIVE_EOF if we hit end of data, ARCHIVE_OK otherwise.
  */
 static int
-next_code(struct archive *a, struct private_data *state)
+next_code(struct archive_read *a, struct private_data *state)
 {
 	int code, newcode;
 
@@ -405,7 +413,7 @@ next_code(struct archive *a, struct private_data *state)
 
 	if (code > state->free_ent) {
 		/* An invalid code is a fatal error. */
-		archive_set_error(a, -1, "Invalid compressed data: code %d is larger than max code %d (input address %X)", code, state->free_ent, state->next_in);
+		archive_set_error(&a->archive, -1, "Invalid compressed data");
 		return (ARCHIVE_FATAL);
 	}
 
@@ -449,24 +457,26 @@ next_code(struct archive *a, struct private_data *state)
  * -1 indicates end of available data.
  */
 static int
-getbits(struct archive *a, struct private_data *state, int n)
+getbits(struct archive_read *a, struct private_data *state, int n)
 {
 	int code, ret;
 	static const int mask[] = {
 		0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f, 0x7f, 0xff,
 		0x1ff, 0x3ff, 0x7ff, 0xfff, 0x1fff, 0x3fff, 0x7fff, 0xffff
 	};
-
+	const void *read_buf;
 
 	while (state->bits_avail < n) {
 		if (state->avail_in <= 0) {
-			ret = (a->client_reader)(a, a->client_data,
-			    (const void **)&state->next_in);
+			read_buf = state->next_in;
+			ret = (a->client_reader)(&a->archive, a->client_data,
+			    &read_buf);
+			state->next_in = read_buf;
 			if (ret < 0)
 				return (ARCHIVE_FATAL);
 			if (ret == 0)
 				return (ARCHIVE_EOF);
-			a->raw_position += ret;
+			a->archive.raw_position += ret;
 			state->avail_in = ret;
 		}
 		state->bit_buffer |= *state->next_in++ << state->bits_avail;
