@@ -24,7 +24,7 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_zip.c,v 1.20 2008/01/15 16:27:15 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_read_support_format_zip.c,v 1.21 2008/02/26 07:17:47 kientzle Exp $");
 
 #ifdef HAVE_ERRNO_H
 #include <errno.h>
@@ -385,8 +385,8 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 
 	zip->version = p->version[0];
 	zip->system = p->version[1];
-	zip->flags = le16dec(p->flags);
-	zip->compression = le16dec(p->compression);
+	zip->flags = archive_le16dec(p->flags);
+	zip->compression = archive_le16dec(p->compression);
 	if (zip->compression <
 	    sizeof(compression_names)/sizeof(compression_names[0]))
 		zip->compression_name = compression_names[zip->compression];
@@ -398,11 +398,11 @@ zip_read_file_header(struct archive_read *a, struct archive_entry *entry,
 	zip->mode = 0;
 	zip->uid = 0;
 	zip->gid = 0;
-	zip->crc32 = le32dec(p->crc32);
-	zip->filename_length = le16dec(p->filename_length);
-	zip->extra_length = le16dec(p->extra_length);
-	zip->uncompressed_size = le32dec(p->uncompressed_size);
-	zip->compressed_size = le32dec(p->compressed_size);
+	zip->crc32 = archive_le32dec(p->crc32);
+	zip->filename_length = archive_le16dec(p->filename_length);
+	zip->extra_length = archive_le16dec(p->extra_length);
+	zip->uncompressed_size = archive_le32dec(p->uncompressed_size);
+	zip->compressed_size = archive_le32dec(p->compressed_size);
 
 	(a->decompressor->consume)(a, sizeof(struct zip_file_header));
 
@@ -504,9 +504,9 @@ archive_read_format_zip_read_data(struct archive_read *a,
 					    "Truncated ZIP end-of-file record");
 					return (ARCHIVE_FATAL);
 				}
-				zip->crc32 = le32dec(p + 4);
-				zip->compressed_size = le32dec(p + 8);
-				zip->uncompressed_size = le32dec(p + 12);
+				zip->crc32 = archive_le32dec(p + 4);
+				zip->compressed_size = archive_le32dec(p + 8);
+				zip->uncompressed_size = archive_le32dec(p + 12);
 				(a->decompressor->consume)(a, 16);
 			}
 
@@ -814,8 +814,8 @@ process_extra(const void* extra, struct zip* zip)
 	const char *p = (const char *)extra;
 	while (offset < zip->extra_length - 4)
 	{
-		unsigned short headerid = le16dec(p + offset);
-		unsigned short datasize = le16dec(p + offset + 2);
+		unsigned short headerid = archive_le16dec(p + offset);
+		unsigned short datasize = archive_le16dec(p + offset + 2);
 		offset += 4;
 		if (offset + datasize > zip->extra_length)
 			break;
@@ -827,9 +827,9 @@ process_extra(const void* extra, struct zip* zip)
 		case 0x0001:
 			/* Zip64 extended information extra field. */
 			if (datasize >= 8)
-				zip->uncompressed_size = le64dec(p + offset);
+				zip->uncompressed_size = archive_le64dec(p + offset);
 			if (datasize >= 16)
-				zip->compressed_size = le64dec(p + offset + 8);
+				zip->compressed_size = archive_le64dec(p + offset + 8);
 			break;
 		case 0x5455:
 		{
@@ -842,11 +842,12 @@ process_extra(const void* extra, struct zip* zip)
 			{
 #ifdef DEBUG
 				fprintf(stderr, "mtime: %lld -> %d\n",
-				    (long long)zip->mtime, le32dec(p + offset));
+				    (long long)zip->mtime,
+				    archive_le32dec(p + offset));
 #endif
 				if (datasize < 4)
 					break;
-				zip->mtime = le32dec(p + offset);
+				zip->mtime = archive_le32dec(p + offset);
 				offset += 4;
 				datasize -= 4;
 			}
@@ -854,7 +855,7 @@ process_extra(const void* extra, struct zip* zip)
 			{
 				if (datasize < 4)
 					break;
-				zip->atime = le32dec(p + offset);
+				zip->atime = archive_le32dec(p + offset);
 				offset += 4;
 				datasize -= 4;
 			}
@@ -862,7 +863,7 @@ process_extra(const void* extra, struct zip* zip)
 			{
 				if (datasize < 4)
 					break;
-				zip->ctime = le32dec(p + offset);
+				zip->ctime = archive_le32dec(p + offset);
 				offset += 4;
 				datasize -= 4;
 			}
@@ -872,12 +873,13 @@ process_extra(const void* extra, struct zip* zip)
 			/* Info-ZIP Unix Extra Field (type 2) "Ux". */
 #ifdef DEBUG
 			fprintf(stderr, "uid %d gid %d\n",
-			    le16dec(p + offset), le16dec(p + offset + 2));
+			    archive_le16dec(p + offset),
+			    archive_le16dec(p + offset + 2));
 #endif
 			if (datasize >= 2)
-				zip->uid = le16dec(p + offset);
+				zip->uid = archive_le16dec(p + offset);
 			if (datasize >= 4)
-				zip->gid = le16dec(p + offset + 2);
+				zip->gid = archive_le16dec(p + offset + 2);
 			break;
 		default:
 			break;
