@@ -1,13 +1,12 @@
 /*-
- * Copyright (c) 2003-2004 Tim Kientzle
+ * Copyright (c) 2003-2007 Tim Kientzle
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
  * are met:
  * 1. Redistributions of source code must retain the above copyright
- *    notice, this list of conditions and the following disclaimer
- *    in this position and unchanged.
+ *    notice, this list of conditions and the following disclaimer.
  * 2. Redistributions in binary form must reproduce the above copyright
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
@@ -25,18 +24,24 @@
  */
 
 #include "archive_platform.h"
-__FBSDID("$FreeBSD: src/lib/libarchive/archive_write_set_format_shar.c,v 1.11 2004/11/05 05:26:30 kientzle Exp $");
+__FBSDID("$FreeBSD: src/lib/libarchive/archive_write_set_format_shar.c,v 1.18 2007/05/29 01:00:19 kientzle Exp $");
 
-#include <sys/stat.h>
+#ifdef HAVE_ERRNO_H
 #include <errno.h>
+#endif
 #include <stdarg.h>
 #include <stdio.h>
+#ifdef HAVE_STDLIB_H
 #include <stdlib.h>
+#endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 
 #include "archive.h"
 #include "archive_entry.h"
 #include "archive_private.h"
+#include "archive_write_private.h"
 
 struct shar {
 	int			 dump;
@@ -53,29 +58,30 @@ struct shar {
 	struct archive_string	 work;
 };
 
-static int	archive_write_shar_finish(struct archive *);
-static int	archive_write_shar_header(struct archive *,
+static int	archive_write_shar_finish(struct archive_write *);
+static int	archive_write_shar_destroy(struct archive_write *);
+static int	archive_write_shar_header(struct archive_write *,
 		    struct archive_entry *);
-static ssize_t	archive_write_shar_data_sed(struct archive *,
+static ssize_t	archive_write_shar_data_sed(struct archive_write *,
 		    const void * buff, size_t);
-static ssize_t	archive_write_shar_data_uuencode(struct archive *,
+static ssize_t	archive_write_shar_data_uuencode(struct archive_write *,
 		    const void * buff, size_t);
-static int	archive_write_shar_finish_entry(struct archive *);
-static int	shar_printf(struct archive *, const char *fmt, ...);
+static int	archive_write_shar_finish_entry(struct archive_write *);
+static int	shar_printf(struct archive_write *, const char *fmt, ...);
 static void	uuencode_group(struct shar *);
 
 static int
-shar_printf(struct archive *a, const char *fmt, ...)
+shar_printf(struct archive_write *a, const char *fmt, ...)
 {
 	struct shar *shar;
 	va_list ap;
 	int ret;
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 	va_start(ap, fmt);
 	archive_string_empty(&(shar->work));
 	archive_string_vsprintf(&(shar->work), fmt, ap);
-	ret = ((a->compression_write)(a, shar->work.s, strlen(shar->work.s)));
+	ret = ((a->compressor.write)(a, shar->work.s, strlen(shar->work.s)));
 	va_end(ap);
 	return (ret);
 }
@@ -84,17 +90,18 @@ shar_printf(struct archive *a, const char *fmt, ...)
  * Set output format to 'shar' format.
  */
 int
-archive_write_set_format_shar(struct archive *a)
+archive_write_set_format_shar(struct archive *_a)
 {
+	struct archive_write *a = (struct archive_write *)_a;
 	struct shar *shar;
 
 	/* If someone else was already registered, unregister them. */
-	if (a->format_finish != NULL)
-		(a->format_finish)(a);
+	if (a->format_destroy != NULL)
+		(a->format_destroy)(a);
 
-	shar = malloc(sizeof(*shar));
+	shar = (struct shar *)malloc(sizeof(*shar));
 	if (shar == NULL) {
-		archive_set_error(a, ENOMEM, "Can't allocate shar data");
+		archive_set_error(&a->archive, ENOMEM, "Can't allocate shar data");
 		return (ARCHIVE_FATAL);
 	}
 	memset(shar, 0, sizeof(*shar));
@@ -103,6 +110,7 @@ archive_write_set_format_shar(struct archive *a)
 	a->pad_uncompressed = 0;
 	a->format_write_header = archive_write_shar_header;
 	a->format_finish = archive_write_shar_finish;
+	a->format_destroy = archive_write_shar_destroy;
 	a->format_write_data = archive_write_shar_data_sed;
 	a->format_finish_entry = archive_write_shar_finish_entry;
 	a->archive_format = ARCHIVE_FORMAT_SHAR_BASE;
@@ -117,12 +125,13 @@ archive_write_set_format_shar(struct archive *a)
  * and other extended file information.
  */
 int
-archive_write_set_format_shar_dump(struct archive *a)
+archive_write_set_format_shar_dump(struct archive *_a)
 {
+	struct archive_write *a = (struct archive_write *)_a;
 	struct shar *shar;
 
-	archive_write_set_format_shar(a);
-	shar = a->format_data;
+	archive_write_set_format_shar(&a->archive);
+	shar = (struct shar *)a->format_data;
 	shar->dump = 1;
 	a->format_write_data = archive_write_shar_data_uuencode;
 	a->archive_format = ARCHIVE_FORMAT_SHAR_DUMP;
@@ -131,16 +140,15 @@ archive_write_set_format_shar_dump(struct archive *a)
 }
 
 static int
-archive_write_shar_header(struct archive *a, struct archive_entry *entry)
+archive_write_shar_header(struct archive_write *a, struct archive_entry *entry)
 {
 	const char *linkname;
 	const char *name;
 	char *p, *pp;
 	struct shar *shar;
-	const struct stat *st;
 	int ret;
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 	if (!shar->wrote_header) {
 		ret = shar_printf(a, "#!/bin/sh\n");
 		if (ret != ARCHIVE_OK)
@@ -156,22 +164,21 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 		archive_entry_free(shar->entry);
 	shar->entry = archive_entry_clone(entry);
 	name = archive_entry_pathname(entry);
-	st = archive_entry_stat(entry);
 
 	/* Handle some preparatory issues. */
-	switch(st->st_mode & S_IFMT) {
-	case S_IFREG:
+	switch(archive_entry_filetype(entry)) {
+	case AE_IFREG:
 		/* Only regular files have non-zero size. */
 		break;
-	case S_IFDIR:
+	case AE_IFDIR:
 		archive_entry_set_size(entry, 0);
 		/* Don't bother trying to recreate '.' */
 		if (strcmp(name, ".") == 0  ||  strcmp(name, "./") == 0)
 			return (ARCHIVE_OK);
 		break;
-	case S_IFIFO:
-	case S_IFCHR:
-	case S_IFBLK:
+	case AE_IFIFO:
+	case AE_IFCHR:
+	case AE_IFBLK:
 		/* All other file types have zero size in the archive. */
 		archive_entry_set_size(entry, 0);
 		break;
@@ -179,7 +186,7 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 		archive_entry_set_size(entry, 0);
 		if (archive_entry_hardlink(entry) == NULL &&
 		    archive_entry_symlink(entry) == NULL) {
-			archive_set_error(a, ARCHIVE_ERRNO_MISC,
+			archive_set_error(&a->archive, ARCHIVE_ERRNO_MISC,
 			    "shar format cannot archive this");
 			return (ARCHIVE_WARN);
 		}
@@ -190,7 +197,7 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 	if (ret != ARCHIVE_OK)
 		return (ret);
 
-	if (!S_ISDIR(st->st_mode)) {
+	if (archive_entry_filetype(entry) != AE_IFDIR) {
 		/* Try to create the dir. */
 		p = strdup(name);
 		pp = strrchr(p, '/');
@@ -201,11 +208,14 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 			/* Try to avoid a lot of redundant mkdir commands. */
 			if (strcmp(p, ".") == 0) {
 				/* Don't try to "mkdir ." */
+				free(p);
 			} else if (shar->last_dir == NULL) {
 				ret = shar_printf(a,
 				    "mkdir -p %s > /dev/null 2>&1\n", p);
-				if (ret != ARCHIVE_OK)
+				if (ret != ARCHIVE_OK) {
+					free(p);
 					return (ret);
+				}
 				shar->last_dir = p;
 			} else if (strcmp(p, shar->last_dir) == 0) {
 				/* We've already created this exact dir. */
@@ -217,11 +227,15 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 			} else {
 				ret = shar_printf(a,
 				    "mkdir -p %s > /dev/null 2>&1\n", p);
-				if (ret != ARCHIVE_OK)
+				if (ret != ARCHIVE_OK) {
+					free(p);
 					return (ret);
+				}
 				free(shar->last_dir);
 				shar->last_dir = p;
 			}
+		} else {
+			free(p);
 		}
 	}
 
@@ -236,8 +250,8 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 		if (ret != ARCHIVE_OK)
 			return (ret);
 	} else {
-		switch(st->st_mode & S_IFMT) {
-		case S_IFREG:
+		switch(archive_entry_filetype(entry)) {
+		case AE_IFREG:
 			if (archive_entry_size(entry) == 0) {
 				/* More portable than "touch." */
 				ret = shar_printf(a, "test -e \"%s\" || :> \"%s\"\n", name, name);
@@ -268,7 +282,7 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 				shar->outbytes = 0;
 			}
 			break;
-		case S_IFDIR:
+		case AE_IFDIR:
 			ret = shar_printf(a, "mkdir -p %s > /dev/null 2>&1\n",
 			    name);
 			if (ret != ARCHIVE_OK)
@@ -287,19 +301,19 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 			 * up at end of archive.
 			 */
 			break;
-		case S_IFIFO:
+		case AE_IFIFO:
 			ret = shar_printf(a, "mkfifo %s\n", name);
 			if (ret != ARCHIVE_OK)
 				return (ret);
 			break;
-		case S_IFCHR:
+		case AE_IFCHR:
 			ret = shar_printf(a, "mknod %s c %d %d\n", name,
 			    archive_entry_rdevmajor(entry),
 			    archive_entry_rdevminor(entry));
 			if (ret != ARCHIVE_OK)
 				return (ret);
 			break;
-		case S_IFBLK:
+		case AE_IFBLK:
 			ret = shar_printf(a, "mknod %s b %d %d\n", name,
 			    archive_entry_rdevmajor(entry),
 			    archive_entry_rdevminor(entry));
@@ -316,18 +330,18 @@ archive_write_shar_header(struct archive *a, struct archive_entry *entry)
 
 /* XXX TODO: This could be more efficient XXX */
 static ssize_t
-archive_write_shar_data_sed(struct archive *a, const void *buff, size_t n)
+archive_write_shar_data_sed(struct archive_write *a, const void *buff, size_t n)
 {
 	struct shar *shar;
 	const char *src;
 	int ret;
 	size_t written = n;
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 	if (!shar->has_data)
 		return (0);
 
-	src = buff;
+	src = (const char *)buff;
 	ret = ARCHIVE_OK;
 	shar->outpos = 0;
 	while (n-- > 0) {
@@ -340,7 +354,7 @@ archive_write_shar_data_sed(struct archive *a, const void *buff, size_t n)
 		shar->outbuff[shar->outpos++] = *src++;
 
 		if (shar->outpos > sizeof(shar->outbuff) - 2) {
-			ret = (a->compression_write)(a, shar->outbuff,
+			ret = (a->compressor.write)(a, shar->outbuff,
 			    shar->outpos);
 			if (ret != ARCHIVE_OK)
 				return (ret);
@@ -349,7 +363,7 @@ archive_write_shar_data_sed(struct archive *a, const void *buff, size_t n)
 	}
 
 	if (shar->outpos > 0)
-		ret = (a->compression_write)(a, shar->outbuff, shar->outpos);
+		ret = (a->compressor.write)(a, shar->outbuff, shar->outpos);
 	if (ret != ARCHIVE_OK)
 		return (ret);
 	return (written);
@@ -380,7 +394,7 @@ uuencode_group(struct shar *shar)
 }
 
 static ssize_t
-archive_write_shar_data_uuencode(struct archive *a, const void *buff,
+archive_write_shar_data_uuencode(struct archive_write *a, const void *buff,
     size_t length)
 {
 	struct shar *shar;
@@ -388,10 +402,10 @@ archive_write_shar_data_uuencode(struct archive *a, const void *buff,
 	size_t n;
 	int ret;
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 	if (!shar->has_data)
 		return (ARCHIVE_OK);
-	src = buff;
+	src = (const char *)buff;
 	n = length;
 	while (n-- > 0) {
 		if (shar->uuavail == 3)
@@ -412,13 +426,13 @@ archive_write_shar_data_uuencode(struct archive *a, const void *buff,
 }
 
 static int
-archive_write_shar_finish_entry(struct archive *a)
+archive_write_shar_finish_entry(struct archive_write *a)
 {
 	const char *g, *p, *u;
 	struct shar *shar;
 	int ret;
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 	if (shar->entry == NULL)
 		return (0);
 
@@ -497,7 +511,7 @@ archive_write_shar_finish_entry(struct archive *a)
 }
 
 static int
-archive_write_shar_finish(struct archive *a)
+archive_write_shar_finish(struct archive_write *a)
 {
 	struct shar *shar;
 	int ret;
@@ -507,7 +521,7 @@ archive_write_shar_finish(struct archive *a)
 	 * fix them all up at end-of-archive.
 	 */
 
-	shar = a->format_data;
+	shar = (struct shar *)a->format_data;
 
 	/*
 	 * Only write the end-of-archive markers if the archive was
@@ -520,12 +534,21 @@ archive_write_shar_finish(struct archive *a)
 		if (ret != ARCHIVE_OK)
 			return (ret);
 		/* Shar output is never padded. */
-		archive_write_set_bytes_in_last_block(a, 1);
+		archive_write_set_bytes_in_last_block(&a->archive, 1);
 		/*
 		 * TODO: shar should also suppress padding of
 		 * uncompressed data within gzip/bzip2 streams.
 		 */
 	}
+	return (ARCHIVE_OK);
+}
+
+static int
+archive_write_shar_destroy(struct archive_write *a)
+{
+	struct shar *shar;
+
+	shar = (struct shar *)a->format_data;
 	if (shar->entry != NULL)
 		archive_entry_free(shar->entry);
 	if (shar->last_dir != NULL)
