@@ -3,10 +3,13 @@ DEFINE_TEST(test_acl_pax)
 DEFINE_TEST(test_archive_api_feature)
 DEFINE_TEST(test_bad_fd)
 DEFINE_TEST(test_compat_gtar)
+DEFINE_TEST(test_compat_tar_hardlink)
 DEFINE_TEST(test_compat_zip)
 DEFINE_TEST(test_empty_write)
 DEFINE_TEST(test_entry)
 DEFINE_TEST(test_entry_strmode)
+DEFINE_TEST(test_link_resolver)
+DEFINE_TEST(test_pax_filename_encoding)
 DEFINE_TEST(test_read_compress_program)
 DEFINE_TEST(test_read_data_large)
 DEFINE_TEST(test_read_extract)
